// parallel.go - run tasks in parallel
package main
import ("fmt"; "time"; "sync")

var wg sync.WaitGroup

func task(tm int) {
    fmt.Printf("Task %d: working...\n", tm)
    time.Sleep(time.Duration(tm) * time.Second)
}

func main() {
    wg.Add(4)
    for n := 1; n <= 4; n++ {
        go func(num int) {
            defer wg.Done()
            task(num)
            fmt.Printf("Task %d: done\n", num)
        }(n)
    }
    wg.Wait()
}

/*************************************

    $ time go run parallel.go
    Task 4: working...
    Task 1: working...
    Task 2: working...
    Task 3: working...
    Task 1: done
    Task 2: done
    Task 3: done
    Task 4: done

    real    0m4.010s
    user    0m0.001s
    sys     0m0.006s

*/
